<?php

namespace NoccyLabs\TinyEvent;

class EventDispatcher implements EventEmitterInterface
{
    protected $listeners = [];

    public function addListener($type, callable $handler, ...$userdata)
    {
        if (!array_key_exists($type, $this->listeners)) {
            $this->listeners[$type] = null;
        }

        $this->listeners[$type][] = (object)[
            'type' => $type,
            'handler' => $handler,
            'data' => (array)$userdata
        ];
    }

    public function dispatchEvent($type, Event $event=null)
    {
        if (!array_key_exists($type, $this->listeners)) {
            return;
        }

        if (empty($event)) {
            $event = new Event($type);
        } else {
            $event->setType($type);
        }

        foreach ($this->listeners[$type] as $listener) {
            $ret = call_user_func($listener->handler, $event, ...$listener->data);
            if ($event->isPropagationStopped() || ($ret === true)) {
                break;
            }
        }
    }
}
